﻿namespace Project.BaseFramework.ORM
{
    using System;
    using System.Collections.Generic;
    using System.Data;
    using System.Data.Common;
    using System.Data.Sql;
    using System.Data.SqlClient;
    using System.Linq;
    using System.Text;

    using Project.BaseFramework.DataProvider;
    using System.Reflection;

    public class DataAccess<T> where T : Model
    {
        private ModelMapping modelMapping = null;

        public DataAccess(string modelName)
        {
            if (SchemaDict.Keys.Contains(modelName))
                this.modelMapping = SchemaDict[modelName]; ////set the model handler
            else
                throw new ArgumentOutOfRangeException();
        }

        private static Dictionary<string, ModelMapping> SchemaDict;
        static DataAccess() 
        {
            ModelMappingReflector reflector = new ModelMappingReflector();
            SchemaDict = reflector.GetModelMapDict();
        }

        public int Add(T model)
        {
            if (model == null)
            {
                throw new ArgumentNullException();
            }

            StringBuilder cmdText = new StringBuilder();
            cmdText.Append("INSERT INTO ").Append(modelMapping.TableName).Append(" (");
            cmdText.Append(string.Join(",", model.ColumnValues.Keys.ToArray()));
            cmdText.Append(") VALUES (@");
            cmdText.Append(string.Join(", @", model.ColumnValues.Keys.ToArray()));
            cmdText.Append(")");

            List<DbParameter> dbParms = new List<DbParameter>();
            foreach (var pair in model.ColumnValues) 
            {
                dbParms.Add(DBHelper.CreateInDbParameter(string.Format("@{0}",pair.Key), modelMapping.ColumnDict[pair.Key].DbDataBype, pair.Value));
            }

            return this.ExecuteNonQuery(CommandType.Text, cmdText.ToString(), dbParms.ToArray());
        }

        public int Update(T model)
        {
            if (model == null)
            {
                throw new ArgumentNullException();
            }

            StringBuilder cmdText = new StringBuilder();
            cmdText.Append("UPDATE ").Append(modelMapping.TableName).Append(" SET ");
            cmdText.Append(ORMUtility.GetUpdateStrWithConnector(model.ColumnValues.Keys, modelMapping.PKColumns));
            cmdText.Append(" WHERE ");
            cmdText.Append(ORMUtility.GetStrWithConnector(modelMapping.PKColumns, " {0}=@{0} AND", 3));

            List<DbParameter> dbParms = new List<DbParameter>();
            foreach (var pair in model.ColumnValues)
            {
                dbParms.Add(DBHelper.CreateInDbParameter(string.Format("@{0}", pair.Key), modelMapping.ColumnDict[pair.Key].DbDataBype, pair.Value));
            }
            return this.ExecuteNonQuery(CommandType.Text, cmdText.ToString(), dbParms.ToArray());
        }

        public int Delete(string pk)
        {
            if (string.IsNullOrEmpty(pk))
            {
                throw new ArgumentNullException();
            }

            string pkName=modelMapping.PKColumns[0];// Support one pk only
            string cmdText = string.Format("DELETE FROM {0} WHERE {1}=@{1}", modelMapping.TableName, pkName);

            DbParameter dbParam = DBHelper.CreateInDbParameter(pkName, modelMapping.ColumnDict[pkName].DbDataBype, pk);

            return this.ExecuteNonQuery(CommandType.Text, cmdText, dbParam);
        }

        public int DeleteByCondition(string condition)
        {
            string cmdText = string.Format("DELETE FROM {0} WHERE {1}", this.modelMapping.TableName, condition);

            return this.ExecuteNonQuery(CommandType.Text, cmdText);
        }

        public T GetModel(string pk)
        {
            if (string.IsNullOrEmpty(pk))
            {
                throw new ArgumentNullException();
            }

            string pkName=modelMapping.PKColumns[0];// Support one pk only
            string cmdText = string.Format("SELECT * FROM {0} WHERE {1}=@{1}", modelMapping.TableName, pkName);

            DbParameter dbParms = DBHelper.CreateInDbParameter(pkName, modelMapping.ColumnDict[pkName].DbDataBype, pk);

            using (DbDataReader dr = DBHelper.ExecuteReader(CommandType.Text, cmdText, dbParms))
            {
                if (dr.Read())
                {
                    return this.GetModel(dr, false);
                }
                return null;
            }
        }

        public T GetModelByCondition(string condition)
        {
            string cmdText = string.Format("SELECT * FROM {0} WHERE {1}", this.modelMapping.TableName, condition);
            using (DbDataReader dr = this.ExecuteReader(CommandType.Text, cmdText))
            {
                if (dr.Read())
                {
                    return this.GetModel(dr,false);
                }
                return null;
            }
        }

        public List<T> GetList()
        {
            string cmdText = string.Format("SELECT * FROM {0}", this.modelMapping.TableName);
            using (DbDataReader dr = this.ExecuteReader(CommandType.Text, cmdText))
            {
                List<T> lst = GetList(dr);
                return lst;
            }
        }

        public List<T> GetList(string condition)
        {
            string cmdText = string.Format("SELECT * FROM {0} WHERE {1}", this.modelMapping.TableName, condition);
            using (DbDataReader dr = this.ExecuteReader(CommandType.Text, cmdText))
            {
                List<T> lst = GetList(dr);
                return lst;
            }
        }

        public List<T> GetListBySQL(string cmdText)
        {
            using (DbDataReader dr = this.ExecuteReader(CommandType.Text,cmdText))
            {
                List<T> lst = GetList(dr);
                return lst;
            }
        }

        public List<T> GetListPaging(string cmdText, int pageIndex, int pageSize, string orderInfo)
        {
            using (DbDataReader dr = this.ExecutePagingReader(CommandType.Text, cmdText,pageIndex,pageSize,orderInfo))
            {
                List<T> lst = GetList(dr);
                return lst;
            }
        } 

        private List<T> GetList(DbDataReader dr)
        {
            List<T> lst = new List<T>();
            while (dr.Read())
            {
                lst.Add(this.GetModel(dr,true));
            }

            return lst;
        }

        private T GetModel(DbDataReader dr, bool isModelInList)
        {
            /*
             * 如果启用此处代码，可以将ModelMapping中的CreateNewModel去掉。
             * 相对于直接用New创建对象，反射创建是慢一些，但一百万次也没到0.2秒；
             * 此方法用的时间是直接创建对象的三倍时间,可能是我测试时系统对象比较少的原因吧。
            */
            //T model = model = Activator.CreateInstance<T>();
            
            /*
             * 此方法是，定义一个返回基类的接口，在各个类中创建对象并转成基类。
             * 在GetModel中，再将基类转成特定的子类；
             * 此方法相对于直接创建对象，一百万次大概多了0.001-0.002秒；
            */
            T model =(T)modelMapping.CreateNewModel();

            model.IsModelInList = isModelInList;
            model.SetDrToModel(dr);
            return model;
        }

        #region DB Helper Method
        public int ExecuteNonQuery(CommandType cmdType, string cmdText, params DbParameter[] cmdParms)
        {
            return DBHelper.ExecuteNonQuery(cmdType, cmdText, cmdParms);
        }

        public object ExecuteScalar(CommandType cmdType, string cmdText, params DbParameter[] cmdParms)
        {
            return DBHelper.ExecuteScalar(cmdType, cmdText, cmdParms);
        }

        public DbDataReader ExecuteReader(CommandType cmdType, string cmdText, params DbParameter[] cmdParms)
        {
            return DBHelper.ExecuteReader(cmdType, cmdText, cmdParms);
        }

        public DataSet ExecuteDataSet(CommandType cmdType, string cmdText, params DbParameter[] cmdParms)
        {
            return DBHelper.ExecuteDataSet(cmdType, cmdText, cmdParms);
        }

        public DataTable ExecuteQuery(CommandType cmdType, string cmdText, params DbParameter[] cmdParms)
        {
            return DBHelper.ExecuteDataTable(cmdType, cmdText, cmdParms);
        }

        public DataTable ExecutePagingDataTable(CommandType cmdType, string cmdText, int pageIndex, int pageSize, string orderInfo, params DbParameter[] parameterValues)
        {
            return DBHelper.ExecutePagingDataTable(cmdType, cmdText, pageIndex, pageSize, orderInfo, parameterValues);
        }

        public DbDataReader ExecutePagingReader(CommandType cmdType, string cmdText, int pageIndex, int pageSize, string orderInfo, params DbParameter[] parameterValues)
        {
            return DBHelper.ExecutePagingReader(cmdType, cmdText, pageIndex, pageSize, orderInfo, parameterValues);
        }

        public void BeginTransaction()
        {
            DBHelper.BeginTransaction();
        }

        public void CommitTransaction()
        {
            DBHelper.CommitTransaction();
        }

        public void RollbackTransaction()
        {
            DBHelper.RollbackTransaction();
        }

        #endregion
    }
}
